/* Copyright (c) 2012 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.nhnent.iap.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import com.nhnent.iap.google.R;
import com.nhnent.iap.sample.service.AuthServiceProvider;
import com.nhnent.iap.sample.service.IAPServiceProvider;
import com.nhnent.iap.sample.service.ItemProvider;
import com.nhnent.mobill.api.core.InAppConfigurationManager;
import com.nhnent.mobill.api.core.InAppMarket;
import com.nhnent.mobill.api.core.InAppServer;
import com.nhnent.mobill.util.ApplicationUtils;
import com.nhnent.mobill.util.ObjectUtils;
import com.toast.android.iap.InAppPurchases;
import com.toast.android.iap.exception.InAppPurchaseException;

public class MainActivity extends Activity implements MainEventListener {
    private static final long ALPHA_GOOGLE_APP_ID = 1000031;
    private static final long ALPHA_TSTORE_APP_ID = 1000275;
    private static final long REAL_GOOGLE_APP_ID = 1000199;
    private static final long REAL_TSTORE_APP_ID = 1000182;

    public static final String TAG = "IAP TEST";
    
    // Handle event
    static final int HANDLE_GIVE_ITEM = 1;
    static final int HANDLE_MOBILL_ERROR = -1;
    static final int HANDLE_HSP_ERROR = -2;

    // Graphics for the gas gauge
    static int[] TANK_RES_IDS = { R.drawable.gas0, R.drawable.gas1, R.drawable.gas2,
                                   R.drawable.gas3, R.drawable.gas4 };

    /**
     * Configuration activity life cycle.
     */
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        
        updateUi();
        
        /**
         * Enable Debug Mode IAP SDK, Available to show Logcat
         */
        InAppPurchases.InAppPurchase.setDebugMode(true);
        
        AuthServiceProvider.initialize(this);
        IAPServiceProvider.initialize(this);

        showAppEnvironment();
    }
    
    // User clicked drive button.
    public void onDriveButtonClicked(View arg0) {
    	if (!AuthServiceProvider.isLogin()) {
    		alert("please login.");
    		return;
    	}
    	
    	if (ItemProvider.getTankCount(this) <= 0) {
    		alert("please charge gas!");
    		return;
    	}
    	
        ItemProvider.updateTankCount(this, ItemProvider.getTankCount(this) - 1);
        updateUi();
        
        alert("Let's go play-museum!");
        Log.d(TAG, "Vrooom. Tank is now " + ItemProvider.getTankCount(this));
    }
    
    // User clicked the "Buy Gas" button
    public void onBuyGasButtonClicked(View arg0) {
    	if (!AuthServiceProvider.isLogin()) {
    		alert("Please login.");
    		return;
    	}
    	
        if (ItemProvider.getTankCount(this) >= ItemProvider.TANK_MAX) {
            complain("your tank is full. drive around a bit!");
            return;
        }

        String itemId = ((EditText) findViewById(R.id.item_id_et)).getText().toString();
        if (ObjectUtils.isEmpty(itemId)) {
        	alert("Insert item ID");
        	return;
        }
        
        // launch the gas purchase UI flow.
        // We will be notified of completion via mPurchaseFinishedListener
        setWaitScreen(true);
        Log.d(TAG, "launching purchase flow for gas.");

        // TODO: request purchase.
        IAPServiceProvider.purchase(this, Long.valueOf(itemId));
    }

    public void onClickedLogin(View arg0) {
    	String userId = ((EditText) findViewById(R.id.register_user_et)).getText().toString();
    	if (ObjectUtils.isEmpty(userId)) {
    		alert("Insert user ID");
    		return;
    	}
    	
    	setWaitScreen(true);
    	AuthServiceProvider.login(this, userId);
    }

    public void onClickedPurchaseList(View arg0) {
    	if (!AuthServiceProvider.isLogin()) {
    		alert("Please login.");
    		return;
    	}
    	
    	setWaitScreen(true);
    	IAPServiceProvider.queryIncompletedPurchases(this);
    }

    public void onClickedQueryItems(View arg0) {
        IAPServiceProvider.queryItems(this);
    }

    public void onClickedProcessesIncompletePurchases(View arg0) {
        if (!AuthServiceProvider.isLogin()) {
            alert("please login.");
            return;
        }

        IAPServiceProvider.processesIncompletePurchases(this);
    }

    public void onClickedChangeEnvBtn(View arg0) {
        if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.GG) {
            IAPServiceProvider.registerAppId(InAppConfigurationManager.getInstance().isRelease() ? REAL_TSTORE_APP_ID : ALPHA_TSTORE_APP_ID);
            IAPServiceProvider.registerMarketId(InAppMarket.TS.getMarketId());
        } else if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.TS) {
            IAPServiceProvider.registerAppId(InAppConfigurationManager.getInstance().isRelease() ? REAL_GOOGLE_APP_ID : ALPHA_GOOGLE_APP_ID);
            IAPServiceProvider.registerMarketId(InAppMarket.GG.getMarketId());
        }

        showAppEnvironment();
    }

    public void onClickedChangePhaseBtn(View arg0) {
        if (InAppConfigurationManager.getInstance().getServer() == InAppServer.REAL) {
            IAPServiceProvider.registerServerPhase(InAppServer.ALPHA);

            if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.GG) {
                IAPServiceProvider.registerAppId(ALPHA_GOOGLE_APP_ID);
            } else if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.TS) {
                IAPServiceProvider.registerAppId(ALPHA_TSTORE_APP_ID);
            }
        } else if (InAppConfigurationManager.getInstance().getServer() == InAppServer.ALPHA) {
            IAPServiceProvider.registerServerPhase(InAppServer.REAL);

            if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.GG) {
                IAPServiceProvider.registerAppId(REAL_GOOGLE_APP_ID);
            } else if (InAppConfigurationManager.getInstance().getMarket() == InAppMarket.TS) {
                IAPServiceProvider.registerAppId(REAL_TSTORE_APP_ID);
            }
        }
        showAppEnvironment();
    }

    public void onClickedQueryTCConsoleItems(View arg0) {
        IAPServiceProvider.queryTCConsoleItems(this);
    }
    
    @Override
	public void updateItemStatus(int receivedItemCount) {
    	Log.d(TAG, "updateItemStatus > " + receivedItemCount);
    	for (int i = 0; i < receivedItemCount ; i++) {
    		int mTank = ItemProvider.getTankCount(MainActivity.this);
			mTank = mTank == ItemProvider.TANK_MAX ? ItemProvider.TANK_MAX : mTank + 1;
	        ItemProvider.updateTankCount(MainActivity.this, mTank);
    	}
				
        updateUi();
        setWaitScreen(false);
	}
    
    @Override
   	public void onInAppPurchaseException(InAppPurchaseException exception) {
   		alert("errorCode=" + exception.getErrorCode() + "\nmessage=" + exception.getMessage());
   		setWaitScreen(false);
   	}
    
    @Override
    public void onAuth(String userId) {
        // TODO: Register user identifier
        IAPServiceProvider.registerUserId(userId);

        // TODO: Query purchase list (Needs consume in server side)
        IAPServiceProvider.queryIncompletedPurchases(this);

        Toast.makeText(MainActivity.this, "Authentication Completed. userId = " + userId, Toast.LENGTH_SHORT).show();
        setWaitScreen(false);
    }

    private void showAppEnvironment() {
    	Bundle bundle = ApplicationUtils.getAndroidManifestMetaData(this);

        long appId = 0;
        String market = "";
        String server = "";

        if (InAppConfigurationManager.getInstance().getMarket() != null) {
            appId = InAppConfigurationManager.getInstance().getAppId();
            market = InAppConfigurationManager.getInstance().getMarket().getMarketId();
        } else if (bundle != null && bundle.get("com.toast.iap.config.appId") != null) {
            appId = (Integer) bundle.get("com.toast.iap.config.appId");
            market = bundle.getString("com.toast.iap.config.market");
        }

        if (InAppConfigurationManager.getInstance().getServer() != null) {
            server = InAppConfigurationManager.getInstance().getServer().name();
        } else if (bundle != null) {
            server = bundle.getString("com.toast.iap.config.server");
        }
    	
    	StringBuilder appInfo = new StringBuilder();
    	appInfo.append("SdkVersion=");
    	appInfo.append(InAppPurchases.SDK_VERSION);
    	appInfo.append(", AppId=");
    	appInfo.append(appId);
    	
    	StringBuilder buildType = new StringBuilder();
    	buildType.append("Market=");
    	buildType.append(market);
    	buildType.append(", BuildType=");
    	buildType.append(server == null ? "REAL" : server);
    	
    	((TextView) findViewById(R.id.sample_app_info_tv)).setText(appInfo.toString());
    	((TextView) findViewById(R.id.sample_build_type_tv)).setText(buildType.toString());
    }
    
    // updates UI to reflect model
    public void updateUi() {
        // update the car color to reflect premium status or lack thereof
        ((ImageView) findViewById(R.id.free_or_premium)).setImageResource(R.drawable.free);

        // "Upgrade" button is only visible if the user is not premium
        findViewById(R.id.register_user_btn).setVisibility(View.VISIBLE);

        // update gas gauge to reflect tank status
        int mTank = ItemProvider.getTankCount(this);
        int index = mTank >= TANK_RES_IDS.length ? TANK_RES_IDS.length - 1 : mTank;
        ((ImageView) findViewById(R.id.gas_gauge)).setImageResource(TANK_RES_IDS[index]);
    }

    // Enables or disables the "please wait" screen.
    void setWaitScreen(boolean set) {
        findViewById(R.id.screen_main).setVisibility(set ? View.GONE : View.VISIBLE);
        findViewById(R.id.screen_wait).setVisibility(set ? View.VISIBLE : View.GONE);
    }

    void complain(String message) {
        alert("Error: " + message);
    }

    void alert(String message) {
        AlertDialog.Builder bld = new AlertDialog.Builder(this);
        bld.setMessage(message);
        bld.setNeutralButton("OK", null);
        Log.d(TAG, "Showing alert dialog: " + message);
        bld.create().show();
    }

}
